import argparse
import sys
import tempfile
from typing import Dict, List

import onnx
from onnxsim import simplify


def has_fixed_shape(shape: onnx.TensorShapeProto) -> bool:
    return all([s.HasField("dim_value") for s in shape.dim])


def check_inputs(inputs: List[onnx.ValueInfoProto]) -> None:
    invalid_inputs = []

    for i in inputs:
        if not has_fixed_shape(i.type.tensor_type.shape):
            invalid_inputs.append(i)

    if len(invalid_inputs) != 0:
        names = " ".join([i.name for i in invalid_inputs])
        print(f"Error: Fix the shape of the inputs by --fix-input-shape option: {names}", file=sys.stderr)
        sys.exit(1)


def fix_input_shapes(shapes: Dict[str, List[int]], model: onnx.ModelProto) -> None:
    for i in model.graph.input:
        if i.name in shapes:
            i.type.tensor_type.shape.ClearField("dim")
            for d in shapes[i.name]:
                dim = i.type.tensor_type.shape.dim.add()
                dim.dim_value = d


def collect_unknown_value(graph: onnx.GraphProto) -> List[str]:
    value_names = [i.name for i in graph.input]
    unknown_names = []
    value_names = value_names + [i.name for i in graph.initializer]
    for n in graph.node:
        for i in n.input:
            if i not in value_names:
                unknown_names.append(i)
        value_names = value_names + list(n.output)
    return unknown_names


def lookup_initializer(name: str, initializer: List[onnx.TensorProto]) -> onnx.TensorProto:
    for i in initializer:
        if i.name == name:
            return i

    raise Exception(f"Error: not found {name}")


def fix_subgraph(graph: onnx.GraphProto, initializer: List[onnx.TensorProto]) -> None:
    unknown_names = collect_unknown_value(graph)
    for name in unknown_names:
        v = lookup_initializer(name, initializer)
        graph.initializer.insert(0, v)


def fix_subgraphs(graph: onnx.GraphProto) -> None:
    for n in graph.node:
        sub_graphs = [attr.g for attr in n.attribute if attr.HasField("g")]
        if len(sub_graphs) != 0:
            for g in sub_graphs:
                fix_subgraph(g, graph.initializer)


def validate_value_names(input_names: List[str], output_names: List[str], model: onnx.ModelProto) -> None:
    names = input_names + output_names
    value_names = (
        [v.name for v in model.graph.value_info] + [v.name for v in model.graph.input] + [v.name for v in model.graph.output]
    )

    if all([n in value_names for n in names]):
        return
    else:
        not_found_names = ", ".join([n for n in names if n not in value_names])
        sys.exit(f"Error: not found the values: {not_found_names}")


def onnigiri(
    input_path: str, output_path: str, input_names: List[str], output_names: List[str], shapes: Dict[str, List[int]]
) -> None:
    model = onnx.load(input_path)
    if len(shapes) != 0:
        fix_input_shapes(shapes, model)
    check_inputs(model.graph.input)

    simplified_model, check = simplify(model)
    assert check, "Error: Simplified ONNX model could not be validated"

    fix_subgraphs(simplified_model.graph)

    validate_value_names(input_names, output_names, simplified_model)

    with tempfile.NamedTemporaryFile() as tmp:
        onnx.save(simplified_model, tmp.name)
        onnx.utils.extract_model(tmp.name, output_path, input_names, output_names)


def parse_shapes(args: List[str]) -> Dict[str, List[int]]:
    shapes = dict()
    if len(args) % 2 != 0:
        sys.exit("Error: the number of argumets of --fix-input-shape must be even.")
    for idx in range(len(args) // 2):
        shapes[args[2 * idx]] = [int(x) for x in args[2 * idx + 1].split(",")]
    return shapes


def main() -> None:
    parser = argparse.ArgumentParser()
    parser.add_argument("input_path", help="Input ONNX path")
    parser.add_argument("-o", "--output", dest="output_path", required=True, help="Output ONNX path")
    parser.add_argument("--from", nargs="+", dest="input_names", help="Names of input value")
    parser.add_argument("--to", nargs="+", dest="output_names", help="Names of output value")
    parser.add_argument("--fix-input-shape", nargs="+", dest="shapes", default=[], help="Pairs of the name and shape of inputs")
    args = parser.parse_args()
    onnigiri(args.input_path, args.output_path, args.input_names, args.output_names, parse_shapes(args.shapes))


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['onnigiri']

package_data = \
{'': ['*']}

install_requires = \
['onnx-simplifier>=0.4,<0.5', 'onnx>=1.9.0,<2.0.0', 'onnxruntime>=1.2.0,<2.0.0']

entry_points = \
{'console_scripts': ['onnigiri = onnigiri.main:main']}

setup_kwargs = {
    'name': 'onnigiri',
    'version': '0.3.1',
    'description': 'onnx-divider',
    'long_description': "# onnigiri\nonnx-divider\n\nThe purpose of this package is to create subgraphs by partitioning computational graphs in order to facilitate the development of applications.\n\nOne of the problems in developing applications using deep learning models is that the DL model is not applicable by itself.\nFor example, they may be have unnecessary nodes and some nodes are not supported some DL tools.\nThis tool enable us to edit an onnx model freely and easily.\n\n## Installation\nFrom [PyPI](https://pypi.org/project/onnigiri/):\n\n```\n$ pip3 install onnigiri\n```\n\nFrom [Dockerhub](https://hub.docker.com/repository/docker/idein/onnigiri)\n\n```\n$ docker pull idein/onnigiri:YYYYMMDD # YYYYMMDD is released date.\nor\n$ docker pull idein/onnigiri:latest\n```\n\n\n## Usage\n[SSD](https://github.com/onnx/models/tree/main/vision/object_detection_segmentation/ssd)\n\n```\n$ onnigiri ssd-10.onnx -o ssd-10-main.onnx --from image --to Transpose_472 Transpose_661\n$ onnigiri ssd-10.onnx -o ssd-10-post.onnx --from Transpose_472 Transpose_661 --to bboxes labels scores\n```\n\nWith docker:\n\n```\n$ docker run --rm -it -u $UID:$GID -v $(pwd):/work idein/onnigiri:latest ssd-10.onnx -o ssd-10-main.onnx --from image --to Transpose_472 Transpose_661\n$ docker run --rm -it -u $UID:$GID -v $(pwd):/work idein/onnigiri:latest ssd-10.onnx -o ssd-10-post.onnx  --from Transpose_472 Transpose_661 --to bboxes labels scores\n```\n\n[UltraFace](https://github.com/onnx/models/tree/main/vision/body_analysis/ultraface)\n\n```\n$ onnigiri version-RFB-640.onnx -o version-RFB-640-main.onnx --from input --to 460 scores\n$ onnigiri version-RFB-640.onnx -o version-RFB-640-post.onnx --from 460 --to boxes\n```\n\n[tiny-yolov3](https://github.com/onnx/models/tree/main/vision/object_detection_segmentation/tiny-yolov3)\n\n```\n$ onnigiri tiny-yolov3-11.onnx --fix-input-shape 'input_1' '1,3,256,256' 'image_shape' '1,2' -o tiny-yolov3-11-main.onnx --from input_1 --to 'TFNodes/yolo_evaluation_layer_1/Reshape_3:0' 'model_1/leaky_re_lu_10/LeakyRelu:0' 'model_1/leaky_re_lu_5/LeakyRelu:0'\n$ onnigiri tiny-yolov3-11.onnx --fix-input-shape 'input_1' '1,3,256,256' 'image_shape' '1,2' -o tiny-yolov3-11-post.onnx --from image_shape 'TFNodes/yolo_evaluation_layer_1/Reshape_3:0' 'model_1/leaky_re_lu_10/LeakyRelu:0' 'model_1/leaky_re_lu_5/LeakyRelu:0' --to 'yolonms_layer_1' 'yolonms_layer_1:1' 'yolonms_layer_1:2'\n```\n\n## Q&A\n\n- How to get the name of values?\n\nUse [Netron](https://netron.app).\n\n- Why is the extracted subgraph different from the original subgraph?\n\nonnigiri apply [onnx-simplifier](https://github.com/daquexian/onnx-simplifier) before extraction.\n\n## Development Guide\n\n```\n$ poetry install\n```\n\n### Build docker image\n\n```\n$ nix build '.#dockerimage' -o image\n$ docker load < ./image\n```\n\n## Related project\n\n- [onnion](https://github.com/Idein/onnion)\n",
    'author': 'Idein Inc.',
    'author_email': 'None',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/Idein/onnigiri',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
